/*
Tecnisa - Tecnologia em Informtica

Nome: GERA_NFRELACCDRS

Descrio:
   Procedimento para relacionamento do item da nota ao CDRS separando a quantidade do item de
   acordo com o valor

CREATE TABLE NFRELACCDRS
(
    CDFILIAL     VARCHAR2(4)            NOT NULL,
    NRAPURACFAT  VARCHAR2(10)           NOT NULL,
    CDCLIENTE    VARCHAR2(21)           NOT NULL,
    NRITEMNF     VARCHAR2(4)            NOT NULL,
    NRLANCTONF   VARCHAR2(10)           NOT NULL,
    CDSERVFILI   VARCHAR2(5)            NULL    ,
    CDPRODUTO    VARCHAR2(10)           NULL    ,
    NRCDRS       VARCHAR2(10)           NULL    ,
    QTEFETIVOS   FLOAT                  NULL    ,
    VRPRECOUNIT  FLOAT                  NULL    ,
    CONSTRAINT PK_NFRELACCDRS PRIMARY KEY (CDFILIAL, NRAPURACFAT, CDCLIENTE, NRITEMNF, NRLANCTONF)
)

*/


CREATE OR REPLACE PROCEDURE GERA_NFRELACCDRS(
         P_CDFILIAL    IN FILIAL.CDFILIAL%TYPE,
         P_NRLANCTONF  IN NOTAFISCAL.NRLANCTONF%TYPE,
         P_NRAGRUEFET  IN EFETCDRS.NRAGRUEFET%TYPE,
         P_NRAGRUPRAT  IN RELAT_CDRS.NRAGRUPRAT%TYPE,
         P_CDFILIAPUR  IN FILIAL.CDFILIAL%TYPE,
         P_CDCLIENTE   IN CLIENTE.CDCLIENTE%TYPE,
         P_DATAINI     IN DATE,
         P_DATAFIM     IN DATE,
         P_NRAPURACFAT in NOTAFISCAL.NRAPURACFAT%TYPE,
         P_IDGERAREL   IN VARCHAR)

AS
   V_SOMAQT       FLOAT;
   V_NRSEQITPROD  VARCHAR2(04);
   V_NRITEM       VARCHAR2(04);
   V_GERANRSEQ    NUMBER;
BEGIN

   IF P_IDGERAREL = 'S' THEN

      V_NRSEQITPROD := '0001';

      FOR C_RELAT_AGRUP IN (SELECT 0 AS QT1, MAX(R.CDFILIAL) AS CDFILIAL, MAX(R.CDCLIENTE) AS CDCLIENTE,
                                   MAX(P.NMPRODUTO) AS NMPRODUTO,
                                   R.CDSERVFILI, MAX(R.CDPRODUTO) AS CDPRODUTO,
                                   NVL(SUM(R.QTEFETIVOS01),0) AS QTEFETIVOS01, NVL(SUM(R.QTEFETIVOS02),0) AS QTEFETIVOS02, NVL(SUM(R.QTEFETIVOS03),0) AS QTEFETIVOS03,
                                   NVL(SUM(R.QTEFETIVOS04),0) AS QTEFETIVOS04, NVL(SUM(R.QTEFETIVOS05),0) AS QTEFETIVOS05, NVL(SUM(R.QTEFETIVOS06),0) AS QTEFETIVOS06,
                                   NVL(SUM(R.QTEFETIVOS07),0) AS QTEFETIVOS07, NVL(SUM(R.QTEFETIVOS08),0) AS QTEFETIVOS08, NVL(SUM(R.QTEFETIVOS09),0) AS QTEFETIVOS09,
                                   NVL(SUM(R.QTEFETIVOS10),0) AS QTEFETIVOS10, NVL(SUM(R.QTEFETIVOS11),0) AS QTEFETIVOS11, NVL(SUM(R.QTEFETIVOS12),0) AS QTEFETIVOS12,
                                   NVL(SUM(R.QTEFETIVOS13),0) AS QTEFETIVOS13, NVL(SUM(R.QTEFETIVOS14),0) AS QTEFETIVOS14, NVL(SUM(R.QTEFETIVOS15),0) AS QTEFETIVOS15,
                                   NVL(SUM(R.QTEFETIVOS16),0) AS QTEFETIVOS16,
                                   VRPRECUNIT01,VRPRECUNIT02,VRPRECUNIT03,VRPRECUNIT04,VRPRECUNIT05,VRPRECUNIT06,VRPRECUNIT07,
                                   VRPRECUNIT08,VRPRECUNIT09,VRPRECUNIT10, VRPRECUNIT11, VRPRECUNIT12, VRPRECUNIT13,VRPRECUNIT14,
                                   VRPRECUNIT15,VRPRECUNIT16, NRCDRS
                            FROM RELAT_CDRS R, PRODUTO P
                            WHERE (R.CDFILIAL    = P_CDFILIAPUR)             AND
                                  (R.CDCLIENTE   = P_CDCLIENTE)              AND
                                  ((R.DTCDRS BETWEEN P_DATAINI AND P_DATAFIM) OR
                                  (R.DTINIAPLASER BETWEEN P_DATAINI AND P_DATAFIM)) AND
                                  (R.NRAGRUPNF   = P_NRAGRUEFET)    AND
                                  (R.NRAGRUPRAT  = P_NRAGRUPRAT)  AND
                                  (R.CDPRODUTO   = P.CDPRODUTO)
                            GROUP BY R.CDSERVFILI,
                                   VRPRECUNIT01,VRPRECUNIT02,VRPRECUNIT03,VRPRECUNIT04,VRPRECUNIT05,VRPRECUNIT06,VRPRECUNIT07,
                                   VRPRECUNIT08,VRPRECUNIT09,VRPRECUNIT10, VRPRECUNIT11, VRPRECUNIT12, VRPRECUNIT13,VRPRECUNIT14,
                                   VRPRECUNIT15,VRPRECUNIT16, NRCDRS
                            ORDER BY CDSERVFILI, CDPRODUTO) LOOP

            SELECT LPAD((TO_NUMBER(NVL(MAX(NRITEMNF),'0000')) + 1), 4, 0)
            INTO V_NRITEM
            FROM NFRELACCDRS
            WHERE CDFILIAL    = P_CDFILIAL AND
                  NRLANCTONF  = P_NRLANCTONF AND
                  NRAPURACFAT = P_NRAPURACFAT;

         FOR C_RELAT_PRECO IN (SELECT DISTINCT VRPRECEFET
                               FROM EFETCDRS
                               WHERE (CDFILIAL = P_CDFILIAPUR)   AND
                                     (CDCLIENTE   = P_CDCLIENTE) AND
                                     (DTEFETCDRS BETWEEN P_DATAINI AND P_DATAFIM) AND
                                     (CDSERVFILI = C_RELAT_AGRUP.CDSERVFILI))LOOP


            V_SOMAQT := 0;


            IF C_RELAT_PRECO.VRPRECEFET = C_RELAT_AGRUP.VRPRECUNIT01 THEN
               V_SOMAQT := V_SOMAQT + C_RELAT_AGRUP.QTEFETIVOS01;
            END IF;

            IF C_RELAT_PRECO.VRPRECEFET = C_RELAT_AGRUP.VRPRECUNIT02   THEN
               V_SOMAQT := V_SOMAQT + C_RELAT_AGRUP.QTEFETIVOS02;
            END IF;

            IF C_RELAT_PRECO.VRPRECEFET = C_RELAT_AGRUP.VRPRECUNIT03 THEN
               V_SOMAQT := V_SOMAQT + C_RELAT_AGRUP.QTEFETIVOS03;
            END IF;

            IF C_RELAT_PRECO.VRPRECEFET = C_RELAT_AGRUP.VRPRECUNIT04 THEN
               V_SOMAQT := V_SOMAQT + C_RELAT_AGRUP.QTEFETIVOS04;
            END IF;

            IF C_RELAT_PRECO.VRPRECEFET = C_RELAT_AGRUP.VRPRECUNIT05 THEN
               V_SOMAQT := V_SOMAQT + C_RELAT_AGRUP.QTEFETIVOS05;
            END IF;

            IF C_RELAT_PRECO.VRPRECEFET = C_RELAT_AGRUP.VRPRECUNIT06 THEN
               V_SOMAQT := V_SOMAQT + C_RELAT_AGRUP.QTEFETIVOS06;
            END IF;

            IF C_RELAT_PRECO.VRPRECEFET = C_RELAT_AGRUP.VRPRECUNIT07 THEN
               V_SOMAQT := V_SOMAQT + C_RELAT_AGRUP.QTEFETIVOS07;
            END IF;

            IF C_RELAT_PRECO.VRPRECEFET = C_RELAT_AGRUP.VRPRECUNIT08 THEN
               V_SOMAQT := V_SOMAQT + C_RELAT_AGRUP.QTEFETIVOS08;
            END IF;

            IF C_RELAT_PRECO.VRPRECEFET = C_RELAT_AGRUP.VRPRECUNIT09 THEN
               V_SOMAQT := V_SOMAQT + C_RELAT_AGRUP.QTEFETIVOS09;
            END IF;

            IF C_RELAT_PRECO.VRPRECEFET = C_RELAT_AGRUP.VRPRECUNIT10 THEN
               V_SOMAQT := V_SOMAQT + C_RELAT_AGRUP.QTEFETIVOS10;
            END IF;

            IF C_RELAT_PRECO.VRPRECEFET = C_RELAT_AGRUP.VRPRECUNIT11 THEN
               V_SOMAQT := V_SOMAQT + C_RELAT_AGRUP.QTEFETIVOS11;
            END IF;

            IF C_RELAT_PRECO.VRPRECEFET = C_RELAT_AGRUP.VRPRECUNIT12 THEN
               V_SOMAQT := V_SOMAQT + C_RELAT_AGRUP.QTEFETIVOS12;
            END IF;

            IF C_RELAT_PRECO.VRPRECEFET = C_RELAT_AGRUP.VRPRECUNIT13 THEN
               V_SOMAQT := V_SOMAQT + C_RELAT_AGRUP.QTEFETIVOS13;
            END IF;

            IF C_RELAT_PRECO.VRPRECEFET = C_RELAT_AGRUP.VRPRECUNIT14 THEN
               V_SOMAQT := V_SOMAQT + C_RELAT_AGRUP.QTEFETIVOS14;
            END IF;

            IF C_RELAT_PRECO.VRPRECEFET = C_RELAT_AGRUP.VRPRECUNIT15 THEN
               V_SOMAQT := V_SOMAQT + C_RELAT_AGRUP.QTEFETIVOS15;
            END IF;

            IF C_RELAT_PRECO.VRPRECEFET = C_RELAT_AGRUP.VRPRECUNIT16 THEN
               V_SOMAQT := V_SOMAQT + C_RELAT_AGRUP.QTEFETIVOS16;
            END IF;


            IF V_SOMAQT <> 0 THEN

               INSERT INTO NFRELACCDRS (CDFILIAL, NRLANCTONF, CDPRODUTO, NRSEQITEMNF, QTEFETIVOS, NRCDRS,
                                        VRPRECOUNIT, CDSERVFILI, CDCLIENTE, NRAPURACFAT, NRITEMNF)
               VALUES (P_CDFILIAL, P_NRLANCTONF, C_RELAT_AGRUP.CDPRODUTO,  V_NRSEQITPROD, V_SOMAQT, C_RELAT_AGRUP.NRCDRS,
                       C_RELAT_PRECO.VRPRECEFET, C_RELAT_AGRUP.CDSERVFILI, P_CDCLIENTE, P_NRAPURACFAT, V_NRITEM);

               COMMIT;

               V_NRITEM := LPAD((TO_NUMBER(NVL(V_NRITEM,'0000')) + 1), 4, 0);
               SELECT COUNT(*)
               INTO V_GERANRSEQ
               FROM (SELECT DISTINCT VRPRECEFET
                     FROM EFETCDRS
                     WHERE (CDFILIAL = P_CDFILIAPUR)   AND
                           (CDCLIENTE   = P_CDCLIENTE) AND
                           (DTEFETCDRS BETWEEN P_DATAINI AND P_DATAFIM) AND
                           (CDSERVFILI = C_RELAT_AGRUP.CDSERVFILI));
               IF (V_GERANRSEQ > 1) THEN
                  V_NRSEQITPROD := LPAD((TO_NUMBER(NVL(V_NRSEQITPROD,'0000')) + 1), 4, 0);
               END IF;
            END IF;
         END LOOP;

      END LOOP;
   END IF;
END;
/



-- VERSAO --
BEGIN ATUVEROBJ('GERA_NFRELACCDRS','5.03.009'); END;
/

